@@ -10,5 +10,5 @@ if tf.__version__[0] == "1":
     tf.enable_eager_execution()
 
 from . import nn, utils, data, datasets, layers
-from .data.graph import Graph, BatchGraph
+from .data.graph import Graph, BatchGraph, HeteroDictGraph
 # from .sparse.sparse_adj import SparseAdj
