@@ -4,6 +4,9 @@ import tf_sparse as tfs
 from tf_geometric.nn.conv.gcn import gcn, gcn_build_cache_for_graph, gcn_build_cache_by_adj
 import tensorflow as tf
 import warnings
+import numpy as np
+
+from tf_geometric.utils.tf_sparse_utils import compute_num_or_size_splits
 
 
 class GCN(tf.keras.Model):
@@ -15,6 +18,10 @@ class GCN(tf.keras.Model):
         x_shape = input_shapes[0]
         num_features = x_shape[-1]
 
+        if self.num_splits is not None:
+            num_h_features = self.units if self.use_kernel else num_features
+            self.num_or_size_splits = compute_num_or_size_splits(num_h_features, self.num_splits)
+
         if self.use_kernel:
             self.kernel = self.add_weight("kernel", shape=[num_features, self.units],
                                           initializer="glorot_uniform", regularizer=self.kernel_regularizer)
@@ -28,15 +35,26 @@ class GCN(tf.keras.Model):
                  norm="both", add_self_loop=True, sym=True,
                  renorm=True, improved=False,
                  edge_drop_rate=0.0,
+                 num_splits=None,
+                 num_or_size_splits=None,
                  kernel_regularizer=None, bias_regularizer=None, *args, **kwargs):
         """
 
         :param units: Positive integer, dimensionality of the output space.
         :param activation: Activation function to use.
         :param use_bias: Boolean, whether the layer uses a bias vector.
+        :param norm: normalization mode both|left|right:
+            - both: (D^(-1/2)A)D^(-1/2);
+            - left: D^(-1/2)A; 
+            - right: AD^(-1/2);
+        :param add_self_loop: Whether add self-loop to adj during normalization.
+        :param sym: Optional, only used when norm=="both". Setting sym=True indicates that the input
+            sparse_adj is symmetric.
         :param renorm: Whether use renormalization trick (https://arxiv.org/pdf/1609.02907.pdf).
         :param improved: Whether use improved GCN or not.
         :param edge_drop_rate: Dropout rate of the propagation weights.
+        :param num_or_size_splits: Split (XW) to compute A(XW) for large graphs (Not affecting the output).
+            See the num_or_size_splits param of the tf.split API.
         :param kernel_regularizer: Regularizer function applied to the `kernel` weights matrix.
         :param bias_regularizer: Regularizer function applied to the bias vector.
         """
@@ -61,6 +79,12 @@ class GCN(tf.keras.Model):
         self.kernel_regularizer = kernel_regularizer
         self.bias_regularizer = bias_regularizer
 
+        if num_splits is not None and num_or_size_splits is not None:
+            raise Exception("cannot provide both num_splits and num_or_size_splits for GCN")
+
+        self.num_splits = num_splits
+        self.num_or_size_splits = num_or_size_splits
+
     def build_cache_by_adj(self, sparse_adj, override=False, cache=None):
         """
         Manually compute the normed edge based on this layer's GCN normalization configuration (self.renorm and self.improved) and put it in graph.cache.
@@ -105,7 +129,8 @@ class GCN(tf.keras.Model):
     def call(self, inputs, cache=None, training=None, mask=None):
         """
 
-        :param inputs: List of graph info: [x, edge_index, edge_weight]
+        :param inputs: List of graph info: [x, sparse_adj], [x, edge_index], 
+            or [x, edge_index, edge_weight]
         :param cache: A dict for caching A' for GCN. Different graph should not share the same cache dict.
         :return: Updated node features (x), shape: [num_nodes, units]
         """
@@ -126,4 +151,5 @@ class GCN(tf.keras.Model):
                    norm=self.norm, add_self_loop=self.add_self_loop, sym=self.sym,
                    renorm=self.renorm, improved=self.improved,
                    edge_drop_rate=self.edge_drop_rate,
+                   num_or_size_splits=self.num_or_size_splits,
                    training=training, cache=cache)
