@@ -29,12 +29,19 @@ def _remove_inf_and_nan(x):
     return x
 
 
-def gcn_norm_adj(sparse_adj, norm="both", add_self_loop=True, sym=True, renorm=True, improved=False, cache: dict = None):
+def gcn_norm_adj(sparse_adj: SparseMatrix, norm="both", add_self_loop=True, sym=True,
+                 renorm=True, improved=False, cache: dict = None):
     """
     Compute normed edge (updated edge_index and normalized edge_weight) for GCN normalization.
 
-    :param norm: normalization mode.
-    :param sparse_adj: SparseMatrix, sparse adjacency matrix.
+    :param sparse_adj: tf_sparse.SparseMatrix, sparse adjacency matrix.
+    :param norm: normalization mode both|left|right:
+        - both: (D^(-1/2)A)D^(-1/2);
+        - left: D^(-1/2)A; 
+        - right: AD^(-1/2);
+    :param add_self_loop: Whether add self-loop to adj during normalization.
+    :param sym: Optional, only used when norm=="both". Setting sym=True indicates that the input
+        sparse_adj is symmetric.
     :param renorm: Whether use renormalization trick (https://arxiv.org/pdf/1609.02907.pdf).
     :param improved: Whether use improved GCN or not.
     :param cache: A dict for caching the updated edge_index and normalized edge_weight.
@@ -54,9 +61,17 @@ def gcn_norm_adj(sparse_adj, norm="both", add_self_loop=True, sym=True, renorm=T
 
     fill_weight = 2.0 if improved else 1.0
 
+    # for non-square adj
+    if sparse_adj.shape[0] != sparse_adj.shape[1]:
+        if add_self_loop:
+            raise Exception("cannot set add_self_loop=True for GCN when sparse_adj.shape[0] != sparse_adj.shape[1]")
+        if sym:
+            raise Exception("cannot set sym=True for GCN when sparse_adj.shape[0] != sparse_adj.shape[1]")
+
     if add_self_loop and norm != "both":
         sparse_adj = sparse_adj.add_diag(fill_weight)
 
+    # (D^(-1/2)A)D^(-1/2)
     if norm == "both":
         if add_self_loop and renorm:
             sparse_adj = sparse_adj.add_diag(fill_weight)
@@ -83,6 +98,7 @@ def gcn_norm_adj(sparse_adj, norm="both", add_self_loop=True, sym=True, renorm=T
             normed_sparse_adj = normed_sparse_adj.add_diag(fill_weight)
             # normed_sparse_adj = normed_sparse_adj.add_self_loop(fill_weight=fill_weight)
 
+    # D^(-1/2)A
     elif norm == "left":
         row_deg = sparse_adj.segment_sum(axis=-1)
         row_deg_inv = tf.pow(row_deg, -1)
@@ -92,13 +108,14 @@ def gcn_norm_adj(sparse_adj, norm="both", add_self_loop=True, sym=True, renorm=T
         # D^(-1)A
         normed_sparse_adj = row_deg_inv @ sparse_adj
 
+    # AD^(-1/2)
     elif norm == "right":
         col_deg = sparse_adj.segment_sum(axis=-1)
         col_deg_inv = tf.pow(col_deg, -1)
         col_deg_inv = _remove_inf_and_nan(col_deg_inv)
         col_deg_inv = tfs.diags(col_deg_inv)
 
-        # D^(-1)A
+        # AD^(-1/2)
         normed_sparse_adj = sparse_adj @ col_deg_inv
 
     else:
@@ -207,7 +224,9 @@ def gcn_mapper(repeated_x, neighbor_x, edge_weight=None):
 
 def gcn(x, sparse_adj: SparseMatrix, kernel, bias=None, activation=None,
         norm="both", add_self_loop=True, sym=True,
-        renorm=True, improved=False, edge_drop_rate=0.0, training=False, cache=None):
+        renorm=True, improved=False, edge_drop_rate=0.0,
+        num_or_size_splits=None,
+        training=False, cache=None):
     """
     Functional API for Graph Convolutional Networks.
 
@@ -219,6 +238,8 @@ def gcn(x, sparse_adj: SparseMatrix, kernel, bias=None, activation=None,
     :param renorm: Whether use renormalization trick (https://arxiv.org/pdf/1609.02907.pdf).
     :param improved: Whether use improved GCN or not.
     :param edge_drop_rate: Dropout rate of the propagation weights.
+    :param num_or_size_splits: Split (XW) to compute A(XW) for large graphs (Not affecting the output).
+        See the num_or_size_splits param of the tf.split API.
     :param cache: A dict for caching A' for GCN. Different graph should not share the same cache dict.
         To use @tf_utils.function with gcn, you should cache the noremd edge information before the first call of the gcn.
 
@@ -250,7 +271,15 @@ def gcn(x, sparse_adj: SparseMatrix, kernel, bias=None, activation=None,
         else:
             h = x @ kernel
 
-    h = normed_sparse_adj @ h
+    # if num_or_size_splits is None or 1:
+    #     directly compute A(XW), equivalent to:
+    #     h = normed_sparse_adj @ h
+    # else, split (XW) to compute A(XW) for large graphs:
+    #     this does not affect the output, which is also:
+    #     h = normed_sparse_adj @ h
+    h = normed_sparse_adj.matmul(h, num_or_size_splits=num_or_size_splits)
+
+    # h = normed_sparse_adj @ h
 
     if bias is not None:
         h += bias
