@@ -8,6 +8,8 @@ from tf_geometric.utils.union_utils import convert_union_to_numpy
 from scipy.sparse.linalg import eigs, eigsh
 import scipy.sparse
 
+from tf_sparse import SparseMatrix
+
 
 def convert_edge_index_to_edge_hash(edge_index, num_nodes=None):
     edge_index_is_tensor = tf.is_tensor(edge_index)
@@ -143,7 +145,7 @@ def convert_edge_to_upper(edge_index, edge_props=None, merge_modes=None):
     upper_edge_index = tf.stack([row, col], axis=0)
     upper_edge_index, upper_edge_props = merge_duplicated_edge(upper_edge_index, edge_props, merge_modes)
 
-    if not edge_index_is_tensor:
+    if tf.executing_eagerly() and not edge_index_is_tensor:
         upper_edge_index = upper_edge_index.numpy()
 
     return upper_edge_index, upper_edge_props
@@ -197,14 +199,14 @@ def convert_edge_to_directed(edge_index, edge_props=None, merge_modes=None):
                 else:
                     pure_lower_edge_prop = tf.boolean_mask(upper_edge_prop, non_self_loop_mask)
                     updated_edge_prop = tf.concat([upper_edge_prop, pure_lower_edge_prop], axis=0)
-                    if not tf.is_tensor(edge_prop):
+                    if tf.executing_eagerly() and not tf.is_tensor(edge_prop):
                         updated_edge_prop = updated_edge_prop.numpy()
                 updated_edge_props.append(updated_edge_prop)
     else:
         updated_edge_index = edge_index
         updated_edge_props = edge_props
 
-    if not edge_index_is_tensor:
+    if tf.executing_eagerly() and not edge_index_is_tensor:
         updated_edge_index = updated_edge_index.numpy()
 
     return updated_edge_index, updated_edge_props
@@ -258,10 +260,10 @@ def remove_self_loop_edge(edge_index, edge_weight=None):
     if edge_weight is not None:
         edge_weight = tf.boolean_mask(edge_weight, mask, axis=0)
 
-    if not edge_index_is_tensor:
+    if tf.executing_eagerly() and not edge_index_is_tensor:
         edge_index = edge_index.numpy()
 
-    if edge_weight is not None and not edge_weight_is_tensor:
+    if tf.executing_eagerly() and edge_weight is not None and not edge_weight_is_tensor:
         edge_weight = edge_weight.numpy()
 
     return edge_index, edge_weight
@@ -627,27 +629,42 @@ def to_scipy_sparse_matrix(edge_index, edge_weight=None, num_nodes=None):
 
 class RandomNeighborSampler(object):
     def __init__(self, edge_index, edge_weight=None):
-        self.edge_index = convert_union_to_numpy(edge_index, np.int32)
+
+        edge_index = convert_union_to_numpy(edge_index, np.int32)
         if edge_weight is not None:
-            self.edge_weight = convert_union_to_numpy(edge_weight)
+            edge_weight = convert_union_to_numpy(edge_weight)
         else:
-            self.edge_weight = np.ones([self.edge_index.shape[1]], dtype=np.float32)
-        self.neighbor_dict = {}
+            edge_weight = np.ones([edge_index.shape[1]], dtype=np.float32)
 
-        for (a, b), weight in zip(self.edge_index.T, self.edge_weight):
 
-            if a not in self.neighbor_dict:
-                neighbors = []
-                self.neighbor_dict[a] = neighbors
+        self.num_row_nodes = convert_union_to_numpy(edge_index[0].max() + 1)
+        self.num_col_nodes = convert_union_to_numpy(edge_index[1].max() + 1)
+
+
+        raw_neighbor_dict = {}
+
+        for (a, b), weight in zip(edge_index.T, edge_weight):
+
+            if a not in raw_neighbor_dict:
+                neighbors = [[],  []]
+                raw_neighbor_dict[a] = neighbors
             else:
-                neighbors = self.neighbor_dict[a]
-            neighbors.append((b, weight))
+                neighbors = raw_neighbor_dict[a]
+            neighbors[0].append(b)
+            neighbors[1].append(weight)
+            # neighbors.append((b, weight))
+
+        self.neighbor_dict = {
+            source: [np.array(neighbors[0]), np.array(neighbors[1])]
+            for source, neighbors in raw_neighbor_dict.items()
+        }
 
         self.num_sources = len(self.neighbor_dict)
         self.source_index = sorted(self.neighbor_dict.keys())
-        self.num_neighbors_dict = {node_index: len(neighbors) for node_index, neighbors in self.neighbor_dict.items()}
+        self.num_neighbors_dict = {node_index: len(neighbors[0]) for node_index, neighbors in self.neighbor_dict.items()}
 
-    def sample(self, k=None, ratio=None, central_node_index=None):
+
+    def sample(self, k=None, ratio=None, sampled_node_index=None, padding=False):
         # if k is None and ratio is None:
         #     raise Exception("you should provide either k or ratio")
         # elif k is not None and ratio is not None:
@@ -661,50 +678,127 @@ class RandomNeighborSampler(object):
         else:
             sample_all = False
 
-        central_node_index_is_none = central_node_index is None
 
-        if central_node_index_is_none:
-            central_node_index = self.source_index
+        if sampled_node_index is None:
+            # use_virtual_node_index = False
+
+            sampled_row_index = self.source_index
+            row_virtual_index = sampled_row_index
+
+            # num_sampled_rows = self.num_row_nodes
+            # num_sampled_cols = self.num_col_nodes
+
+        else:
+            # use_virtual_node_index = True
+            if isinstance(sampled_node_index, tuple):
+                sampled_row_index, sampled_col_index = sampled_node_index
+            else:
+                sampled_row_index = sampled_node_index
+                sampled_col_index = sampled_node_index
+
+            row_virtual_mapping = -np.ones([self.num_row_nodes])
+            row_virtual_index = np.arange(0, len(sampled_row_index))
+            row_virtual_mapping[sampled_row_index] = row_virtual_index
+
+            if isinstance(sampled_node_index, tuple):
+                col_virtual_mapping = -np.ones([self.num_col_nodes])
+                col_virtual_index = np.arange(0, len(sampled_col_index))
+                col_virtual_mapping[sampled_col_index] = col_virtual_index
+            else:
+                col_virtual_index = row_virtual_index
+                col_virtual_mapping = row_virtual_mapping
+
+            # num_sampled_rows = len(sampled_row_index)
+            # num_sampled_cols = len(sampled_col_index)
+
 
         # num_central_nodes = len(central_node_index)
 
-        sampled_edge_index = []
-        sampled_edge_weight = []
+        sampled_virtual_edge_index_list = []
+        sampled_virtual_edge_weight_list = []
 
-        for i in central_node_index:
-            if i not in self.neighbor_dict:
+        for row_virtual_i, row_i in zip(row_virtual_index, sampled_row_index):
+            if row_i not in self.neighbor_dict:
                 continue
-            neighbors = self.neighbor_dict[i]
 
-            if sample_all:
-                sampled_neighbors = neighbors
+            neighbor_index, neighbor_weight = self.neighbor_dict[row_i]
+
+            if sampled_node_index is not None:
+                virtual_neighbor_index = col_virtual_mapping[neighbor_index]
+                mask = virtual_neighbor_index >= 0
+                virtual_neighbor_index = virtual_neighbor_index[mask]
+
+                if len(virtual_neighbor_index) == 0:
+                    continue
+
+                virtual_neighbor_weight = neighbor_weight[mask]
+            else:
+                virtual_neighbor_index = neighbor_index
+                virtual_neighbor_weight = neighbor_weight
+
+
+            if sample_all or (ratio is None and not padding and k >= len(virtual_neighbor_index)):
+                sampled_virtual_neighbor_index = virtual_neighbor_index
+                sampled_virtual_neighbor_weight = virtual_neighbor_weight
+
             else:
                 if ratio is None:
                     num_sampled_neighbors = k
+                    replace = padding and k >= len(virtual_neighbor_index)
                 else:
-                    num_sampled_neighbors = np.ceil(self.num_neighbors_dict.get(i, 0) * ratio).astype(np.int32)
+                    num_sampled_neighbors = np.ceil(len(virtual_neighbor_index) * ratio).astype(np.int32)
+                    replace = False
 
-                sampled_neighbor_index = np.random.choice(len(neighbors), num_sampled_neighbors, replace=True)
-                sampled_neighbors = [neighbors[i] for i in sampled_neighbor_index]
+                range_index = np.arange(0, len(virtual_neighbor_index))
+                sampled_index = np.random.choice(range_index, num_sampled_neighbors, replace=replace)
 
-            for (j, weight) in sampled_neighbors:
-                sampled_edge_index.append([i, j])
-                sampled_edge_weight.append(weight)
+                # print(neighbor_index)
 
-        sampled_edge_index = np.array(sampled_edge_index).T
-        sampled_edge_weight = np.array(sampled_edge_weight)
+                sampled_virtual_neighbor_index = virtual_neighbor_index[sampled_index]
+                sampled_virtual_neighbor_weight = virtual_neighbor_weight[sampled_index]
 
-        sampled_node_index = np.unique(np.reshape(sampled_edge_index, [-1]))
-        central_node_index_set = {i for i in central_node_index}
-        non_central_node_index = [i for i in sampled_node_index if i not in central_node_index_set]
-        sampled_node_index = np.concatenate([central_node_index, non_central_node_index], axis=0)
+            sampled_virtual_edge_index = np.stack([np.full(sampled_virtual_neighbor_index.shape, fill_value=row_virtual_i), sampled_virtual_neighbor_index], axis=0)
 
-        sampled_edge = sampled_edge_index, sampled_edge_weight
-        if not central_node_index_is_none:
-            node_index = sampled_node_index, central_node_index, non_central_node_index
-            return sampled_edge, node_index
+            sampled_virtual_edge_index_list.append(sampled_virtual_edge_index)
+            sampled_virtual_edge_weight_list.append(sampled_virtual_neighbor_weight)
+
+        if len(sampled_virtual_edge_index_list) > 0:
+            sampled_virtual_edge_index = np.concatenate(sampled_virtual_edge_index_list, axis=1)
+            sampled_virtual_edge_weight = np.concatenate(sampled_virtual_edge_weight_list, axis=0)
         else:
-            return sampled_edge
+            sampled_virtual_edge_index = None
+            sampled_virtual_edge_weight = None
+
+        return sampled_virtual_edge_index, sampled_virtual_edge_weight
+
+                # sampled_neighbor_index = np.random.choice(len(neighbors), num_sampled_neighbors, replace=replace)
+                # sampled_neighbors = [neighbors[i] for i in sampled_neighbor_index]
+
+            # for (j, weight) in sampled_neighbors:
+            #     sampled_edge_index.append([central, j])
+            #     sampled_edge_weight.append(weight)
+
+        # sampled_edge_index = np.array(sampled_edge_index).T
+        # sampled_edge_weight = np.array(sampled_edge_weight)
+
+        # sampled_virtual_adj = SparseMatrix(
+        #     sampled_virtual_edge_index, 
+        #     value=sampled_virtual_edge_weight,
+        #     shape=[num_sampled_rows, num_sampled_cols])
+
+        # return sampled_virtual_adj
+
+        # sampled_node_index = np.unique(np.reshape(sampled_edge_index, [-1]))
+        # central_node_index_set = {i for i in central_node_index}
+        # non_central_node_index = [i for i in sampled_node_index if i not in central_node_index_set]
+        # sampled_node_index = np.concatenate([central_node_index, non_central_node_index], axis=0)
+        #
+        # sampled_edge = sampled_edge_index, sampled_edge_weight
+        # if not central_node_index_is_none:
+        #     node_index = sampled_node_index, central_node_index, non_central_node_index
+        #     return sampled_edge, node_index
+        # else:
+        #     return sampled_edge
 
 
 class LaplacianMaxEigenvalue(object):
